@@ -165,6 +165,7 @@ type curlHTTPVersion =
   | HTTP_VERSION_1_1
   | HTTP_VERSION_2
   | HTTP_VERSION_2TLS
+  | HTTP_VERSION_2_PRIOR_KNOWLEDGE
 
 type curlDebugType =
   | DEBUGTYPE_TEXT
@@ -245,6 +246,37 @@ type curlProxyType =
   | CURLPROXY_SOCKS4A (** added in 7.18.0 *)
   | CURLPROXY_SOCKS5_HOSTNAME (** added in 7.18.0 *)
 
+type curlMIMEPartData =
+  | CURLMIME_DATA of string
+  | CURLMIME_FILEDATA of string
+
+type curlMIMEEncoding =
+  | CURLMIME_8BIT
+  | CURLMIME_BINARY
+  | CURLMIME_7BIT
+  | CURLMIME_QUOTEDPRINTABLE
+  | CURLMIME_BASE64
+  | CURLMIME_NONE
+
+type curlMIMEPart =
+  {
+    encoding: curlMIMEEncoding;
+    headers: string list;
+    subparts: curlMIMEPart list;
+    data: curlMIMEPartData;
+  }
+
+type curlKHMatch =
+  | CURLKHMATCH_OK
+  | CURLKHMATCH_MISMATCH of string
+  | CURLKHMATCH_MISSING
+
+type curlKHStat =
+  | CURLKHSTAT_FINE_ADD_TO_FILE
+  | CURLKHSTAT_FINE
+  | CURLKHSTAT_REJECT
+  | CURLKHSTAT_DEFER
+
 (** Protocols to enable (via CURLOPT_PROTOCOLS and CURLOPT_REDIR_PROTOCOLS) *)
 type curlProto =
 | CURLPROTO_ALL (** enable everything *)
@@ -424,6 +456,9 @@ type curlOption =
   | CURLOPT_LOGIN_OPTIONS of string
   | CURLOPT_CONNECT_TO of string list
   | CURLOPT_POSTREDIR of curlPostRedir list
+  | CURLOPT_MIMEPOST of curlMIMEPart list
+  | CURLOPT_SSHKNOWNHOSTS of string
+  | CURLOPT_SSHKEYFUNCTION of (curlKHMatch -> string -> curlKHStat)
 
 type initOption =
   | CURLINIT_GLOBALALL
@@ -512,8 +547,10 @@ type version_info = {
 
 external version_info : unit -> version_info = "caml_curl_version_info"
 
-external strerror : curlCode -> string = "caml_curl_easy_strerror"
-let errno : curlCode -> int = Obj.magic
+external strerror : curlCode -> string = "caml_curl_strerror"
+external curlCode_of_int : int -> curlCode option = "caml_curl_curlCode_of_int"
+external int_of_curlCode : curlCode -> int = "caml_curl_int_of_curlCode"
+let errno = int_of_curlCode
 
 type pauseOption = PAUSE_SEND | PAUSE_RECV | PAUSE_ALL
 
@@ -944,6 +981,15 @@ let set_connect_to conn l =
 let set_postredir conn l =
   setopt conn (CURLOPT_POSTREDIR l)
 
+let set_mimepost conn part =
+  setopt conn (CURLOPT_MIMEPOST part)
+
+let set_sshknownhosts conn s =
+  setopt conn (CURLOPT_SSHKNOWNHOSTS s)
+
+let set_sshkeyfunction conn f =
+  setopt conn (CURLOPT_SSHKEYFUNCTION f)
+
 let get_effectiveurl conn =
   match (getinfo conn CURLINFO_EFFECTIVE_URL) with
   | CURLINFO_String s -> s
@@ -1278,6 +1324,9 @@ class handle =
     method set_autoreferer b = set_autoreferer conn b
     method set_opensocketfunction closure = set_opensocketfunction conn closure
     method set_proxytype t = set_proxytype conn t
+    method set_mimepost p = set_mimepost conn p
+    method set_sshknownhosts s = set_sshknownhosts conn s
+    method set_sshkeyfunction f = set_sshkeyfunction conn f
 
     method get_effectiveurl = get_effectiveurl conn
     method get_redirecturl = get_redirecturl conn
